#!/bin/bash
#
# IceWarp Server
# Copyright (c) 2008-2010 IceWarp Ltd. All rights reserved.
#
# http://www.icewarp.com
#
# file: functions.sh - helper functions to start daemons
#

# This file is not to be run directly
if [ `basename "$0"` == "daemonlib.sh" ]; then
    echo "This file must be included from script."
    exit 1
fi

# test if have functions included
if [ -z "$FUNCTIONS_INCLUDED" ]; then
    echo "Script daemonlib.sh requires to have included functions.sh"
    exit 1
fi

# initial variables

# wait time in seconds before sending SIGKILL to daemon
MAX_WAIT_SECONDS="5"
# wait time in micro seconds == 1/5 second
WAIT_TIME="200000"
# max wait time in microseconds == 5 seconds
MAX_WAIT_TIME=$WAIT_TIME
let "MAX_WAIT_TIME*=${MAX_WAIT_SECONDS}*5"

DAEMON_DESC=""
DAEMON=""

# Function returns daemon description string according to name
# Parameters:
#   $1 - name of dameon
# Returns:
#   Sets variables DAEMON_DESC and DAEMON
#   always 0
#
get_daemon_desc()
{
    case "$1" in
	"control")
	    DAEMON_DESC="IceWarp Server Control daemon"
	    DAEMON="control"
	    ;;
	"smtp")
	    DAEMON_DESC="IceWarp Server SMTP daemon"
	    DAEMON="smtp"
	    ;;
	"pop3"|"imap")
	    DAEMON_DESC="IceWarp Server POP3/IMAP4 daemon"
	    DAEMON="pop3"
	    ;;
	"im")
	    DAEMON_DESC="IceWarp Server Messaging daemon"
	    DAEMON="im"
	    ;;
	"cal"|"calendar")
	    DAEMON_DESC="IceWarp Server GroupWare daemon"
	    DAEMON="cal"
	    ;;
	"php"|"phpd")
	    DAEMON_DESC="IceWarp Server PHP"
	    DAEMON="php"
	    ;;
	*)
	    DAEMON_DESC="Unknown daemon '${1}'"
	    DAEMON="unknown"
	    return 1
	    ;;
    esac
    return 0
}

startdaemon()
{
    get_daemon_desc "$1"
    if [ $? -ne 0 ]; then
        echo "ERROR: Can not identify daemon name: ${1}" 1>&2
        exit 1
    fi

    pushd "$IWS_INSTALL_DIR" &>/dev/null

    echo -n "Starting '${DAEMON_DESC}': "
    export IWS_INSTALL_DIR
    export IWS_PROCESS_USER
    export KL_PLUGINS_PATH="${IWS_INSTALL_DIR}/kaspersky/ppl"
    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${IWS_INSTALL_DIR}/avast:${IWS_INSTALL_DIR}/kaspersky:${KL_PLUGINS_PATH}:${LD_LIBRARY_PATH}"

    "${IWS_INSTALL_DIR}/${DAEMON}" "--installdir=${IWS_INSTALL_DIR}" &> /dev/null
    if [ $? -eq 0 ]; then
	    echo "OK"
    else
        echo "FAIL"
    fi

    popd &>/dev/null
    return 0
}

stopdaemon()
{
    get_daemon_desc "$1"
    if [ $? -ne 0 ]; then
	echo "ERROR: Can not identify daemon name: ${1}" 1>&2
	exit 1
    fi

    PID=$(cat "${IWS_INSTALL_DIR}/var/${DAEMON}.pid" 2> /dev/null)

    echo -n "Shutting down '${DAEMON_DESC}', PID ${PID}: "
    if [ "x$PID" == "x" ]; then
	echo "Can not read PID."
	return 0;
    fi

    WAITED="0"
    while [ true ]
    do
	kill $PID 2> /dev/null
	usleep $WAIT_TIME

	kill -0 $PID 2> /dev/null
	if [ $? -ne 0 ]; then
	    break
	fi

	let "WAITED+=$WAIT_TIME"
	if [ "$WAITED" == "$MAX_WAIT_TIME" ]; then
		echo -n "${MAX_WAIT_SECONDS} seconds timeouted, killing: "
		kill -9 $PID
		break
	fi
    done

    rm -f "${IWS_INSTALL_DIR}/var/${DAEMON}.pid" 2> /dev/null
    echo "OK"

    # stop PHP when stopping control service
    if [ "$DAEMON" == "control" ]; then
	stopdaemon "php"
    fi
    return 0
}

startphp()
{
    # get first argument which should be max PHP processes
    PHP_FCGI_CHILDREN="$1"
    # test if PHP_FCGI_CHILDREN is empty string
    if [ "x$PHP_FCGI_CHILDREN" == "x" ]; then
	PHP_FCGI_CHILDREN=8
    fi
    # test if PHP_FCGI_CHILDREN is number
    echo $PHP_FCGI_CHILDREN | grep "[^0-9]" > /dev/null 2>&1
    if [ $? -eq 0 ]; then 
	PHP_FCGI_CHILDREN=8
    fi
    if [ $PHP_FCGI_CHILDREN -eq 0 ]; then
	PHP_FCGI_CHILDREN=8
    fi

    pushd "$IWS_INSTALL_DIR" &> /dev/null

    export IWS_INSTALL_DIR
    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    export ASPELL_PREFIX="${IWS_INSTALL_DIR}/php"

    export PHP_FCGI_MAX_REQUESTS=1000
    export PHP_FCGI_CHILDREN

    # need to save PID of php process
    "${IWS_INSTALL_DIR}/php/php" -c "${IWS_INSTALL_DIR}/php/php.ini" -b "${IWS_INSTALL_DIR}/var/php.socket" 2> "${IWS_INSTALL_DIR}/logs/phpstartup.log" & echo $! > "${IWS_INSTALL_DIR}/var/php.pid"

    popd &> /dev/null
    return 0
}

stopphp()
{
    stopdaemon "php"
    return $?
}

executephp()
{
    if [ "x$1" == "x" ]; then
        return -1
    fi

    pushd "$IWS_INSTALL_DIR" &> /dev/null

    export IWS_INSTALL_DIR
    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    export ASPELL_PREFIX="${IWS_INSTALL_DIR}/php"

    "${IWS_INSTALL_DIR}/php/php" -c "{IWS_INSTALL_DIR}/php/php.ini" $*
    RET_VAL=$?

    popd &> /dev/null
    return $RET_VAL
}

startpurpleserv()
{
    pushd "$IWS_INSTALL_DIR" &> /dev/null

    export IWS_INSTALL_DIR
    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    export LD_LIBRARY_PATH="${IWS_INSTALL_DIR}/purple/lib:${IWS_INSTALL_DIR}/purple/plugins:${LD_LIBRARY_PATH}"

    # need to start in purpleserv directory
    # need to save PID of purpleserv process
    cd "${IWS_INSTALL_DIR}/purple" &> /dev/null
    if [ $? -ne 0 ]; then
    	echo "ERROR: Cannot change to purple directory" &> "${IWS_INSTALL_DIR}/logs/purple/purpleerror.log" 1>&2
    	return 1
    fi
    "${IWS_INSTALL_DIR}/purple/purpleserv" &> "${IWS_INSTALL_DIR}/logs/purple/purplestartup.log" & echo $! > "${IWS_INSTALL_DIR}/var/purpleserv.pid"

    popd &> /dev/null

    return 0
}
