#!/bin/bash
#
# IceWarp Server
# Copyright (c) 2008-2010 IceWarp Ltd. All rights reserved.
#
# http://www.icewarp.com
#
# file: functions.sh - include file with basic functions
#

# This file is not to be run directly
if [ `basename "$0"` == "functions.sh" ]; then
    echo "This file must be included from script." 1>&2
    exit 1
fi

# Test if have functions included
if [ -n "$FUNCTIONS_INCLUDED" ]; then
    echo "This file was already included." 1>&2
    exit 1
fi

colorize()
{
    if [ "$TERM" = "linux" ] ||
       [ "$TERM" = "xterm" ] ||
       [ "$TERM" = "xterm-color" ] ||
       [ "$TERM" = "screen" ] &&
       [ "$USE_COLORS" = "1" ]; then
	    GOOD=$'\e[32;01m'
	    WARN=$'\e[33;01m'
	    BAD=$'\e[31;01m'
	    NORMAL=$'\e[0m'
	    HILITE=$'\e[37;01m'
	    BRACKET=$'\e[34;01m'
	else
	    GOOD=$'\e[0;32m'
	    BAD=$'\e[0;31m'
	    NORMAL=$'\e[0m'
	    WARN=$NORMAL
	    HILITE=$NORMAL
	    BRACKET=$NORMAL
    fi
}

preconf()
{
    source "/etc/icewarp/icewarp.conf"
    if [ $? -ne 0 ]; then
        echo "ERROR: can not source IceWarp Server configuration file: /etc/icewarp/icewarp.conf" 1>&2
        exit 1
    fi

    if [ "x${IWS_INSTALL_DIR}" == "x" ]; then
        echo "ERROR: icewarp.conf: IWS_INSTALL_DIR has invalid value."
        exit 1
    fi
    if [ "x${IWS_PROCESS_USER}" == "x" ]; then
        echo "ERROR: icewarp.conf: IWS_PROCESS_USER has invalid value."
        exit 1
    fi
    if [ "x${IWS_VERSION}" == "x" ]; then
        echo "ERROR: icewarp.conf: IWS_VERSION has invalid value."
        exit 1
    fi

    SERVER_LD_LIBRARY_PATH="${IWS_INSTALL_DIR}/lib:${IWS_INSTALL_DIR}/php/lib"
    FUNCTIONS_INCLUDED="1"

    colorize
}

good()
{
    echo -e "${GOOD}**${NORMAL}\t$*"
}

bad()
{
    echo -e "${BAD}**${NORMAL}\t$*"
}

verybad()
{
    echo "${BAD}************${HILITE}************${BAD}************${NORMAL}"
    echo "${HILITE}$*"
    echo "${BAD}************${HILITE}************${BAD}************${NORMAL}"
    exit 1
}

warn()
{
    echo -e "${WARN}**${NORMAL}\t$*"
}

hilite()
{
    echo -e "${GOOD}**${HILITE}\t$*${NORMAL}"
}

getparam()
{
    echo -e -n "${GOOD}**${NORMAL}\t${1} "
    read PARAM
}

getpassword()
{
    DIFFER="1"
    while [ "$DIFFER" == "1" ]
	do
	    echo -e -n "${GOOD}**${NORMAL}\t${1} "
	    read -s TOKEN
	    good ""
	    echo -e -n "${GOOD}**${NORMAL}\tOnce again to confirm: "
	    read -s TOKEN_CONFIRM
	    good ""

	    if [ "x$TOKEN" == "x$TOKEN_CONFIRM" ]; then
		    PASSWORD_PARAM=$TOKEN
		    DIFFER="0"
		else
		    bad ""
		    bad "Passwords don't match."
		    bad "Try again"
		    bad ""
	    fi
	done
}

# Ask question with confirmation
# Parameters:
# $1 - question
# $2 - first choice, it is choosed if user enters empty string
# $3 - second choice
# Returns:
# 0 on error - unknown string entered
# 1 if first choice was selected
# 2 if second choice was selected
ask_with_confirmation()
{
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]; then
	return 0
    fi
    LOWER_CHOICE_1=$(echo "$2" | tr "A-Z" "a-z")
    LOWER_CHOICE_2=$(echo "$3" | tr "A-Z" "a-z")
    echo -e -n "${GOOD}**${NORMAL}\t${1} [${HILITE}${2}${NORMAL}/${3}]: "
    read PARAM
    PARAM=$(echo "$PARAM" | tr "A-Z" "a-z")
    if [ "x$PARAM" == "x" ] || [ "x$PARAM" == "x$LOWER_CHOICE_1" ]; then
	return 1
    fi
    if [ "x$PARAM" == "x$LOWER_CHOICE_2" ]; then
	return 2
    fi
    return 0
}

display_menu()
{
    STR_LENGTH=${#1}
    STR_LINE=$(while [ "$STR_LENGTH" -ne "0" ]; do printf "%s" "-"; let "STR_LENGTH-=1"; done; printf "\n")
    good ""
    hilite "$1"
    hilite "$STR_LINE"
    good ""
    good "You have the following options:"
    good ""
    # LINE and $2 can contain spaces
    for LINE in `cat ${3} | grep "^${2}:"`
	do
	    NAME=$(echo "$LINE" | cut -d':' -f2)
	    LINECHOICE=$(echo "$LINE" |cut -d':' -f3)
	    display_menuitem "$NAME" "$LINECHOICE"
	done
    good ""
    display_menuitem "0" "Return"
    display_menuitem "Q" "Exit"
    good ""
    getparam "Enter your choice:"
}

# This function expects exported variable MENU_SCRIPTS_DIR with menu scripts stored
menu()
{
    IFS_PREV=$IFS
    IFS=$'\n'
    while true
    do
	display_menu "$1" "$2" "$3"
	# quit from menu
	if [ "x$PARAM" == "x0" ]; then
	    return
	fi
	# quit from script which uses menu
	if [ "x$PARAM" == "xQ" ] || [ "x$PARAM" == "xq" ]; then
	    exit 0
	fi
	for LINE in `cat ${3} | grep "^${2}:"`
	    do
		LINECHOICE=$(echo "$LINE" | cut -d':' -f2)
		NAME=$(echo "$LINE" | cut -d':' -f3)
		TYPE=$(echo "$LINE" | cut -d':' -f4)
		if [ "x$LINECHOICE" == "x$PARAM" ]; then
		    unset PARAM
		    case "$TYPE" in
			"script"|"SCRIPT")
			    good ""
			    SCRIPT=$(echo "$LINE" | cut -d':' -f5)
			    pushd "$IWS_INSTALL_DIR" &>/dev/null
			    export MENU_SCRIPTS_DIR
			    "${MENU_SCRIPTS_DIR}/${SCRIPT}"
			    if [ $? -ne 0 ]; then
				bad "Error running script!"
			    fi
			    popd &>/dev/null
			;;
			"menu"|"MENU")
			    MENUROOT=$(echo "$LINE" | cut -d':' -f5)
			    if [ "$MENUROOT" != "QUIT" ]; then
				clear
				menu "$NAME" "$MENUROOT" "$3"
				clear
			    fi
			;;
			*)
			    MATCHED="0"
			;;
			esac
		fi
	    done # for LINE ...
    done # while true
    IFS=$IFS_PREV
}

display_menuitem()
{
    CHOICE=$1
    NAME=$2
    echo -e "${GOOD}**\t[${HILITE}${CHOICE}${GOOD}]${HILITE} ${NAME}${NORMAL}"
}



run_cmd_log()
{
    # log command
    DATE_TIME=$(date "+%Y-%m-%d %H:%M:%S.%N")
    echo "${DATE_TIME}: \"$1\" \"$2\" \"$3\" \"$4\" \"$5\" \"$6\" \"$7\" \"$8\" \"$9\" \"${10}\" \"${11}\" \"${12}\" \"${13}\" \"${14}\" \"${15}\"" >> "$WIZARD_CMD_LOG"

    # run command, catch only stdout
    # because Merak uses stdout
    # pass parameters one by one, because parameter can contain space
    # example is domain description
    export LD_LIBRARY_PATH="${SERVER_LD_LIBRARY_PATH}:${LD_LIBRARY_PATH}"
    CMD_STDOUT=$("$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9" "${10}" "${11}" "${12}" "${13}" "${14}" "${15}")
    CMD_RET=$?

    # log command output
    DATE_TIME=$(date "+%Y-%m-%d %H:%M:%S.%N")
    echo "${DATE_TIME}: Command output:" >> "$WIZARD_CMD_LOG"

    DATE_TIME=$(date "+%Y-%m-%d %H:%M:%S.%N")
    echo "${DATE_TIME}: ${CMD_STDOUT}" >> "$WIZARD_CMD_LOG"

    # log command result
    DATE_TIME=$(date "+%Y-%m-%d %H:%M:%S.%N")
    echo "${DATE_TIME}: Result: ${CMD_RET}" >> "$WIZARD_CMD_LOG"

    return $CMD_RET
}

run_cmd_print_stdout_log()
{
    CMD_STDOUT=""
    CMD_RET="0"
    run_cmd_log $*
    echo "$CMD_STDOUT"
    return $CMD_RET
}

preconf
